#!/usr/bin/env python3
from flask import Flask
from flask_restful import Api
import sentry_sdk
from sentry_sdk.integrations.flask import FlaskIntegration
from sentry_sdk.integrations.sqlalchemy import SqlalchemyIntegration
from healthcheck import HealthCheck

from errorhandlers import register_all_error_handlers
from config import SENTRY_DSN, RELEASE_ID, RELEASEMODE, SQLALCHEMY_DATABASE_URI, JWT_SECRET_KEY
from db import db
from jwtman import jwtman
from fbcrypt import bcrypt
from marshm import ma
from healthchecks import health_database_status
from resources import SignupApi, LoginApi, UsersApi, UserParameterApi, CreateIngestResource, CreateEncodeResource, \
    CreateRestreamResource, ManipulateStreamResource, GetAllStreamResources, ModifyEncodeResource, \
    ModifyIngressResource, ModifyRestreamResource, CoordModifyResource

"""
Main Flask RESTful API
"""

__author__ = "@tormakris"
__copyright__ = "Copyright 2020, videON Team"
__module_name__ = "app"
__version__text__ = "1"

if SENTRY_DSN:
    sentry_sdk.init(
        dsn=SENTRY_DSN,
        integrations=[FlaskIntegration(), SqlalchemyIntegration()],
        traces_sample_rate=1.0,
        send_default_pii=True,
        release=RELEASE_ID,
        environment=RELEASEMODE,
        _experiments={"auto_enabling_integrations": True}
    )

app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = SQLALCHEMY_DATABASE_URI
app.config['JWT_SECRET_KEY'] = JWT_SECRET_KEY
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

api = Api(app)
health = HealthCheck()
db.init_app(app)
ma.init_app(app)
bcrypt.init_app(app)
jwtman.init_app(app)

api.add_resource(SignupApi, '/api/auth/signup')
api.add_resource(LoginApi, '/api/auth/login')
api.add_resource(UsersApi, '/api/users')
api.add_resource(UserParameterApi, '/api/users/<username>')
api.add_resource(CreateIngestResource, '/api/objects/streamerobjects/ingest')
api.add_resource(CreateEncodeResource, '/api/objects/streamerobjects/encode')
api.add_resource(CreateRestreamResource, '/api/objects/streamerobjects/restream')
api.add_resource(ManipulateStreamResource, '/api/objects/streamerobjects/<resourceid>')
api.add_resource(GetAllStreamResources, '/api/objects/streamerobjects')
api.add_resource(ModifyEncodeResource, '/api/objects/streamerobjects/encode/<resourceid>')
api.add_resource(ModifyIngressResource, '/api/objects/streamerobjects/ingest/<resourceid>')
api.add_resource(ModifyRestreamResource, '/api/objects/streamerobjects/restream/<resourceid>')
api.add_resource(CoordModifyResource, '/api/objects/streamerobjects/coordmodify/<resourceid>')

health.add_check(health_database_status)
app.add_url_rule("/healthz", "healthcheck", view_func=lambda: health.run())

register_all_error_handlers(app)


@app.before_first_request
def init_db():
    db.create_all()
